import logging
from json import loads

from aiohttp import ClientSession

from config import LLM_API_KEY, LLM_BASE_URL, LLM_MODEL_NAME


logger = logging.getLogger("utils.llm")


def parse_to_json(value: str) -> dict | list:
    logger.info(f"Handling '{value}'")

    try:
        s = "```json"
        ind = value.index(s)
        value = value[ind + len(s) :]
    except ValueError:
        pass

    try:
        s = "```"
        ind = value.index(s)
        value = value[:ind]
    except ValueError:
        pass

    logger.info(f"Parsed to {value}")
    return loads(value)


def _create_request():
    http_sess = None

    async def fetch(messages: list[dict], temperature: float = 0.7):
        nonlocal http_sess

        if http_sess is None:
            http_sess = ClientSession(
                base_url=LLM_BASE_URL,
                headers={"Authorization": f"Bearer {LLM_API_KEY}"},
            )

        body = {"model": LLM_MODEL_NAME, "messages": messages, "temperature": temperature}
        rsp = await http_sess.post("chat/completions", json=body)
        rsp.raise_for_status()
        data = await rsp.json()
        return data

    return fetch


fetch_response = _create_request()



from contextlib import asynccontextmanager, contextmanager
from datetime import UTC, datetime
from typing import AsyncGenerator, Generator

from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.orm import sessionmaker, Session

from config import DB_ENGINE, DB_ENGINE_SYNC


smaker = sessionmaker(
    DB_ENGINE,
    class_=AsyncSession,
    autocommit=False,
    autoflush=False,
    expire_on_commit=False,
)
smaker_sync = sessionmaker(
    DB_ENGINE_SYNC, class_=Session, autocommit=False, autoflush=False
)


def get_datetime():
    return datetime.now(UTC)


@asynccontextmanager
async def get_db_sess() -> AsyncGenerator[AsyncSession, None]:
    async with smaker.begin() as s:
        try:
            yield s
        except:
            await s.rollback()
            raise


@contextmanager
def get_db_sess_sync() -> Generator[Session, None, None]:
    with smaker_sync.begin() as s:
        try:
            yield s
        except:
            s.rollback()
            raise



from typing import Any, Generic, Literal, TypeVar
from uuid import UUID, uuid4

from pydantic import Field

from enums import (
    ActionStatus,
    CoreEventType,
    MessagePlatformType,
    ModeratorEventType,
)
from models import CustomBaseModel, Action, MessageContext, MessageEvaluation
from utils.db import get_datetime


A = TypeVar("A", bound=Action)
M = TypeVar("M", bound=MessageContext)
C = TypeVar("C")


class CoreEvent(CustomBaseModel):
    """Generic system event."""

    type: CoreEventType
    data: Any
    id: UUID = Field(default_factory=uuid4)
    timestamp: int = Field(default_factory=lambda: int(get_datetime().timestamp()))


# Moderator Events


class ModeratorEvent(CustomBaseModel):
    """Base deployment event."""

    type: ModeratorEventType
    moderator_id: UUID


class StartModeratorEvent(ModeratorEvent, Generic[C]):
    """Deployment start request."""

    type: ModeratorEventType = ModeratorEventType.START
    moderator_id: UUID
    platform: MessagePlatformType
    conf: C


class StopModeratorEvent(ModeratorEvent):
    """Deployment stop request."""

    type: ModeratorEventType = ModeratorEventType.STOP
    reason: str | None = None


class DeadModeratorEvent(ModeratorEvent):
    """Deployment stopped."""

    type: ModeratorEventType = ModeratorEventType.DEAD
    reason: str | None = None


class HeartbeatModeratorEvent(ModeratorEvent):
    type: ModeratorEventType = ModeratorEventType.HEARTBEAT
    role: Literal["moderator", "server"]
    timestamp: int


class ActionPerformedModeratorEvent(ModeratorEvent, Generic[A, M]):
    """Deployment action event."""

    type: ModeratorEventType = ModeratorEventType.ACTION_PERFORMED
    action_type: Any  # Enum
    params: A
    status: ActionStatus
    context: M
    message_id: UUID


class EvaluationCreatedModeratorEvent(ModeratorEvent, Generic[A, M]):
    """Message evaluation result."""

    type: ModeratorEventType = ModeratorEventType.EVALUATION_CREATED
    message_id: UUID
    evaluation: MessageEvaluation[A]
    context: M


class ErrorModeratorEvent(ModeratorEvent):
    type: ModeratorEventType = ModeratorEventType.ERROR
    stack_trace: str | None = None



from datetime import datetime
from enum import Enum
from json import loads
from typing import Any, Generic, TypeVar
from uuid import UUID

from pydantic import BaseModel, Field

from enums import MessagePlatformType


class CustomBaseModel(BaseModel):
    model_config = {
        "json_encoders": {
            UUID: str,
            datetime: lambda dt: dt.isoformat(),
            Enum: lambda e: e.value,
        }
    }

    def to_serialisable_dict(self) -> dict:
        return loads(self.model_dump_json())


class Action(CustomBaseModel):
    type: Enum
    requires_approval: bool
    reason: str = Field(description="Filled by the system and not the agent.")


class ActionDefinition(CustomBaseModel):
    """
    The client facing model(s) which allow prefilling
    parameters. Used in the platforms config for defining
    the allowed actions.
    """

    type: Enum
    requires_approval: bool


class MessageContext(CustomBaseModel):
    platform: MessagePlatformType
    platform_author_id: str
    platform_message_id: str
    content: str
    metadata: Any


class TopicEvaluation(BaseModel):
    topic: str
    topic_score: float


T = TypeVar("T", bound=Action)


class MessageEvaluation(CustomBaseModel, Generic[T]):
    evaluation_score: float
    topic_evaluations: list[TopicEvaluation]
    action: T | None



from enum import Enum


class PricingTierType(str, Enum):
    FREE = "free"
    PRO = "pro"
    ENTERPRISE = "enterprise"


class MessagePlatformType(str, Enum):
    DISCORD = "discord"


class ModeratorStatus(str, Enum):
    OFFLINE = "offline"
    PENDING = "pending"
    ONLINE = "online"


class ActionStatus(str, Enum):
    FAILED = "failed"
    SUCCESS = "success"
    DECLINED = "declined"
    AWAITING_APPROVAL = "awaiting_approval"
    APPROVED = "approved"


class CoreEventType(str, Enum):
    MODERATOR_EVENT = "moderator_event"


class ModeratorEventType(str, Enum):
    START = "start"
    ALIVE = "alive"
    STOP = "stop"
    DEAD = "dead"
    FAILED = "failed"
    HEARTBEAT = "heartbeat"
    ACTION_PERFORMED = "action"
    EVALUATION_CREATED = "evaluation"
    ERROR = "error"
    WARNING = "warning"


class LogSeverity(str, Enum):
    INFO = "INFO"
    WARNING = "WARNING"
    ERROR = "ERROR"
    CRITICAL = "CRITICAL"



from json import dumps

from pydantic import BaseModel

from core.models import CustomBaseModel


def dump_model(value: CustomBaseModel | BaseModel) -> bytes:
    if isinstance(value, CustomBaseModel):
        return dumps(value.to_serialisable_dict()).encode()
    return value.model_dump_json().encode()